""" Extensions/Install.py """

# Copyright (c) 2005 by eduplone Open Source Business Network EEIG, Austria
#
# Generated: 
# Generator: ArchGenXML Version 1.4.0-RC1 devel
#            http://plone.org/products/archgenxml
#
# German Free Software License (D-FSL), see LICENSE_D-FSL_1_0_en.txt
#
__author__    = '''Jens W. Klein <jens.klein@jensquadrat.com>, Daniel Nouri
<dpunktnpunkt@web.de>'''
__docformat__ = 'plaintext'
__version__   = '$ Revision 0.0 $'[11:-2]

import os.path
import sys
from StringIO import StringIO

from App.Common import package_home
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.utils import manage_addTool
from Products.ExternalMethod.ExternalMethod import ExternalMethod
from zExceptions import NotFound, BadRequest

from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
try:
    from Products.Archetypes.lib.register import listTypes
except ImportError:
    from Products.Archetypes.public import listTypes
from Products.Hephaistos.config import PROJECTNAME
from Products.Hephaistos.config import product_globals as GLOBALS

def install(self):
    """ External Method to install Hephaistos """
    out = StringIO()
    print >> out, "Installation log of %s:" % PROJECTNAME

    # If the config contains a list of dependencies, try to install
    # them.  Add a list called DEPENDENCIES to your custom
    # AppConfig.py (imported by config.py) to use it.
    try:
        from Products.Hephaistos.config import DEPENDENCIES
    except:
        DEPENDENCIES = []
    portal = getToolByName(self,'portal_url').getPortalObject()
    quickinstaller = portal.portal_quickinstaller
    for dependency in DEPENDENCIES:
        print >> out, "Installing dependency %s:" % dependency
        quickinstaller.installProduct(dependency)
        get_transaction().commit(1)

    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    install_subskin(self, out, GLOBALS)

    #autoinstall tools
    portal = getToolByName(self,'portal_url').getPortalObject()
    for t in ['HephaistosFactory']:
        try:
            portal.manage_addProduct[PROJECTNAME].manage_addTool(t)
        except BadRequest:
            # if an instance with the same name already exists this error will
            # be swallowed. Zope raises in an unelegant manner a 'Bad Request' error
            pass
        except:
            e = sys.exc_info()
            if e[0] != 'Bad Request':
                raise
    #hide tools in the navigation
    portalProperties = getToolByName(self, 'portal_properties', None)
    if portalProperties is not None:
        navtreeProperties = getattr(portalProperties, 'navtree_properties', None)
        if navtreeProperties:
            navtreeProperties.idsNotToList = list(navtreeProperties.idsNotToList) + \
                                  [toolname for toolname in [u'portal_hephaistos'] \
                                            if toolname not in navtreeProperties.idsNotToList]
    # register tools as configlets
    portal_controlpanel = getToolByName(self,'portal_controlpanel')
    portal_controlpanel.registerConfiglet(
        'HephaistosFactory', #id of your Tool
        'HephaistosFactory', # Title of your Troduct
        'string:${portal_url}/portal_hephaistos//',
        'python:True', # a condition
        'Manage Portal', # access permission
        'Products', # section to which the configlet should be added: (Plone,Products,Members)
        1, # visibility
        'HephaistosFactoryID',
        'site_icon.gif', # icon in control_panel
        'Configuration for tool HephaistosFactory.',
        None,
    )
##    # set title of tool:
##    tool = getToolByName(self, 'portal_hephaistos')
##    tool.title = 'HephaistosFactory'
##
##    portalTypes = getToolByName(self, 'portal_types', None)
##    if portalTypes is not None:
##        existingTypes = portalTypes.listContentTypes()
##        if 'HephaistosFactory' in existingTypes:
##            portalTypes.manage_delObjects(ids=['HephaistosFactory'])
##
##    portal = getToolByName(self,'portal_url').getPortalObject()
##    objIds = portal.objectIds()
##    if 'portal_hephaistos' in objIds:
##        portal['portal_hephaistos'].unindexObject()


    # try to call a workflow install method
    # in 'InstallWorkflows.py' method 'installWorkflows'
    try:
        installWorkflows = ExternalMethod('temp','temp',PROJECTNAME+'.InstallWorkflows', 'installWorkflows').__of__(self)
    except NotFound:
        installWorkflows = None

    if installWorkflows:
        print >>out,'Workflow Install:'
        res = installWorkflows(self,out)
        print >>out,res or 'no output'
    else:
        print >>out,'no workflow install'


    # enable portal_factory for given types
    factory_tool = getToolByName(self,'portal_factory')
    factory_types=[
        ] + factory_tool.getFactoryTypes().keys()
    factory_tool.manage_setPortalFactoryTypes(listOfTypeIds=factory_types)

    # For plone 2.1, allow the easy registering of stylesheets
    from Products.Hephaistos.config import HAS_PLONE21
    if HAS_PLONE21:
        try:
            from Products.Hephaistos.config import STYLESHEETS
        except:
            STYLESHEETS = []
        try:
            from Products.Hephaistos.config import JAVASCRIPTS
        except:
            JAVASCRIPTS = []
        portal_css = getToolByName(portal, 'portal_css')
        portal_javascripts = getToolByName(portal, 'portal_javascripts')
        for stylesheet in STYLESHEETS:
            try:
                portal_css.unregisterResource(stylesheet['id'])
            except:
                pass
            defaulttitle = '%s %s' % (PROJECTNAME, stylesheet['id'])
            defaults = {'id': '',
            'expression': None,
            'media': 'all',
            'title': defaulttitle,
            'enabled': True}
            defaults.update(stylesheet)
            portal_css.manage_addStylesheet(**defaults)
        for javascript in JAVASCRIPTS:
            try:
                portal_javascripts.unregisterResource(stylesheet['id'])
            except:
                pass
            defaults = {'id': '',
            'expression': '', 
            'inline': False,
            'enabled': True,
            'cookable': True}
            defaults.update(javascript)
            portal_javascripts.registerScript(**defaults)

    # try to call a custom install method
    # in 'AppInstall.py' method 'install'
    try:
        install = ExternalMethod('temp','temp',PROJECTNAME+'.AppInstall', 'install')
    except NotFound:
        install = None

    if install:
        print >>out,'Custom Install:'
        res = install(self)
        if res:
            print >>out,res
        else:
            print >>out,'no output'
    else:
        print >>out,'no custom install'
    return out.getvalue()

def uninstall(self):
    out = StringIO()

    # unhide tools
    portalProperties = getToolByName(self, 'portal_properties', None)
    if portalProperties is not None:
        navtreeProperties = getattr(portalProperties, 'navtree_properties', None)
        if navtreeProperties:
            navtreeProperties.idsNotToList = list(navtreeProperties.idsNotToList)
            for toolname in [toolname for toolname in [u'portal_hephaistos'] \
                                      if toolname not in navtreeProperties.idsNotToList]:
                if toolname in navtreeProperties.idsNotToList:
                    navtreeProperties.idsNotToList.remove(toolname)


    # unregister tools as configlets
    portal_control_panel = getToolByName(self,'portal_controlpanel', None)
    if portal_control_panel is not None:
        portal_control_panel.unregisterConfiglet('HephaistosFactory')
    # try to call a workflow uninstall method
    # in 'InstallWorkflows.py' method 'uninstallWorkflows'
    try:
        installWorkflows = ExternalMethod('temp','temp',PROJECTNAME+'.InstallWorkflows', 'uninstallWorkflows').__of__(self)
    except NotFound:
        installWorkflows = None

    if installWorkflows:
        print >>out,'Workflow Uninstall:'
        res = uninstallWorkflows(self,out)
        print >>out,res or 'no output'
    else:
        print >>out,'no workflow uninstall'

    # try to call a custom uninstall method
    # in 'AppInstall.py' method 'uninstall'
    try:
        uninstall = ExternalMethod('temp','temp',PROJECTNAME+'.AppInstall', 'uninstall')
    except:
        uninstall = None

    if uninstall:
        print >>out,'Custom Uninstall:'
        res = uninstall(self)
        if res:
            print >>out,res
        else:
            print >>out,'no output'
    else:
        print >>out,'no custom uninstall'

    return out.getvalue()
